@@ -25,19 +25,17 @@
 
 /* Field shift amounts for sfont_bank_prog bit field integer */
 #define PROG_SHIFTVAL   0
-#define BANK_SHIFTVAL   7
-#define SFONT_SHIFTVAL  21
+#define BANK_SHIFTVAL   8
+#define SFONT_SHIFTVAL  22
 
 /* Field mask values for sfont_bank_prog bit field integer */
-#define PROG_MASKVAL    0x0000007F
-#define BANK_MASKVAL    0x001FFF80
-#define BANKLSB_MASKVAL 0x00003F80
-#define BANKMSB_MASKVAL 0x001FC000
-#define SFONT_MASKVAL   0xFFE00000
+#define PROG_MASKVAL    0x000000FF      /* Bit 7 is used to indicate unset state */
+#define BANK_MASKVAL    0x003FFF00
+#define BANKLSB_MASKVAL 0x00007F00
+#define BANKMSB_MASKVAL 0x003F8000
+#define SFONT_MASKVAL   0xFFC00000
 
 
-#define SETCC(_c,_n,_v)  _c->cc[_n] = _v
-
 static void fluid_channel_init(fluid_channel_t* chan);
 
 
@@ -55,6 +53,7 @@ new_fluid_channel(fluid_synth_t* synth, int num)
   chan->synth = synth;
   chan->channum = num;
   chan->preset = NULL;
+  chan->shadow_preset = NULL;
   chan->tuning = NULL;
 
   fluid_channel_init(chan);
@@ -76,6 +75,7 @@ fluid_channel_init(fluid_channel_t* chan)
   chan->sfont_bank_prog = 0 << SFONT_SHIFTVAL | banknum << BANK_SHIFTVAL
     | prognum << PROG_SHIFTVAL;
 
+  /* FIXME - fluid_synth_find_preset not real-time safe */
   newpreset = fluid_synth_find_preset(chan->synth, banknum, prognum);
   fluid_channel_set_preset(chan, newpreset);
 
@@ -137,26 +137,26 @@ fluid_channel_init_ctrl(fluid_channel_t* chan, int is_all_ctrl_off)
         continue;
       }
 
-      SETCC(chan, i, 0);
+      fluid_channel_set_cc (chan, i, 0);
     }
   }
   else {
     for (i = 0; i < 128; i++) {
-      SETCC(chan, i, 0);
+      fluid_channel_set_cc (chan, i, 0);
     }
   }
 
   /* Set RPN controllers to NULL state */
-  SETCC(chan, RPN_LSB, 127);
-  SETCC(chan, RPN_MSB, 127);
+  fluid_channel_set_cc (chan, RPN_LSB, 127);
+  fluid_channel_set_cc (chan, RPN_MSB, 127);
 
   /* Set NRPN controllers to NULL state */
-  SETCC(chan, NRPN_LSB, 127);
-  SETCC(chan, NRPN_MSB, 127);
+  fluid_channel_set_cc (chan, NRPN_LSB, 127);
+  fluid_channel_set_cc (chan, NRPN_MSB, 127);
 
   /* Expression (MSB & LSB) */
-  SETCC(chan, EXPRESSION_MSB, 127);
-  SETCC(chan, EXPRESSION_LSB, 127);
+  fluid_channel_set_cc (chan, EXPRESSION_MSB, 127);
+  fluid_channel_set_cc (chan, EXPRESSION_LSB, 127);
 
   if (!is_all_ctrl_off) {
 
@@ -164,19 +164,19 @@ fluid_channel_init_ctrl(fluid_channel_t* chan, int is_all_ctrl_off)
 
     /* Just like panning, a value of 64 indicates no change for sound ctrls */
     for (i = SOUND_CTRL1; i <= SOUND_CTRL10; i++) {
-      SETCC(chan, i, 64);
+      fluid_channel_set_cc (chan, i, 64);
     }
 
     /* Volume / initial attenuation (MSB & LSB) */
-    SETCC(chan, VOLUME_MSB, 100);
-    SETCC(chan, VOLUME_LSB, 0);
+    fluid_channel_set_cc (chan, VOLUME_MSB, 100);
+    fluid_channel_set_cc (chan, VOLUME_LSB, 0);
 
     /* Pan (MSB & LSB) */
-    SETCC(chan, PAN_MSB, 64);
-    SETCC(chan, PAN_LSB, 0);
+    fluid_channel_set_cc (chan, PAN_MSB, 64);
+    fluid_channel_set_cc (chan, PAN_LSB, 0);
 
     /* Reverb */
-    /* SETCC(chan, EFFECTS_DEPTH1, 40); */
+    /* fluid_channel_set_cc (chan, EFFECTS_DEPTH1, 40); */
     /* Note: although XG standard specifies the default amount of reverb to 
        be 40, most people preferred having it at zero.
        See http://lists.gnu.org/archive/html/fluid-dev/2009-07/msg00016.html */
@@ -192,6 +192,7 @@ delete_fluid_channel(fluid_channel_t* chan)
   return FLUID_OK;
 }
 
+/* FIXME - Calls fluid_channel_init() potentially in synthesis context */
 void
 fluid_channel_reset(fluid_channel_t* chan)
 {
@@ -207,6 +208,9 @@ fluid_channel_set_preset(fluid_channel_t* chan, fluid_preset_t* preset)
 
   fluid_preset_notify (chan->preset, FLUID_PRESET_UNSELECTED, chan->channum);
 
+  /* Set shadow preset again, so it contains the actual latest assigned value */
+  fluid_atomic_pointer_set (&chan->shadow_preset, preset);
+
   if (chan->preset)     /* Queue preset free (shouldn't free() in synth context) */
   {
     event = fluid_event_queue_get_inptr (chan->synth->return_queue);
