@@ -37,7 +37,6 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <pthread.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -45,9 +44,6 @@
 
 #define BUFFER_LENGTH 512
 
-/* SCHED_FIFO priorities for OSS threads (see pthread_attr_setschedparam) */
-#define OSS_PCM_SCHED_PRIORITY 90
-#define OSS_MIDI_SCHED_PRIORITY 90
 
 /** fluid_oss_audio_driver_t
  *
@@ -59,7 +55,7 @@ typedef struct {
   fluid_synth_t* synth;
   fluid_audio_callback_t read;
   void* buffer;
-  pthread_t thread;
+  fluid_thread_t *thread;
   int cont;
   int dspfd;
   int buffer_size;
@@ -76,17 +72,15 @@ typedef struct {
 int delete_fluid_oss_audio_driver(fluid_audio_driver_t* p);
 
 /* local utilities */
-static int fluid_oss_get_caps(fluid_oss_audio_driver_t* dev);
 static int fluid_oss_set_queue_size(fluid_oss_audio_driver_t* dev, int ss, int ch, int qs, int bs);
-static int fluid_oss_get_sample_formats(fluid_oss_audio_driver_t* dev);
-static void* fluid_oss_audio_run(void* d);
-static void* fluid_oss_audio_run2(void* d);
+static void fluid_oss_audio_run(void* d);
+static void fluid_oss_audio_run2(void* d);
 
 
 typedef struct {
   fluid_midi_driver_t driver;
   int fd;
-  pthread_t thread;
+  fluid_thread_t *thread;
   int status;
   unsigned char buffer[BUFFER_LENGTH];
   fluid_midi_parser_t* parser;
@@ -97,7 +91,7 @@ new_fluid_oss_midi_driver(fluid_settings_t* settings,
 			 handle_midi_event_func_t handler, void* data);
 int delete_fluid_oss_midi_driver(fluid_midi_driver_t* p);
 int fluid_oss_midi_driver_status(fluid_midi_driver_t* p);
-static void* fluid_oss_midi_run(void* d);
+static void fluid_oss_midi_run(void* d);
 
 
 void
@@ -118,12 +112,9 @@ new_fluid_oss_audio_driver(fluid_settings_t* settings, fluid_synth_t* synth)
   int queuesize;
   double sample_rate;
   int periods, period_size;
-  char* devname;
+  int realtime_prio = 0;
+  char* devname = NULL;
   int format;
-  pthread_attr_t attr;
-  int err;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
 
   dev = FLUID_NEW(fluid_oss_audio_driver_t);
   if (dev == NULL) {
@@ -135,6 +126,7 @@ new_fluid_oss_audio_driver(fluid_settings_t* settings, fluid_synth_t* synth)
   fluid_settings_getint(settings, "audio.periods", &periods);
   fluid_settings_getint(settings, "audio.period-size", &period_size);
   fluid_settings_getnum(settings, "synth.sample-rate", &sample_rate);
+  fluid_settings_getint (settings, "audio.realtime-prio", &realtime_prio);
 
   dev->dspfd = -1;
   dev->synth = synth;
@@ -167,8 +159,13 @@ new_fluid_oss_audio_driver(fluid_settings_t* settings, fluid_synth_t* synth)
     goto error_recovery;
   }
 
-  if (!fluid_settings_getstr(settings, "audio.oss.device", &devname)) {
-    devname = "/dev/dsp";
+  if (!fluid_settings_dupstr(settings, "audio.oss.device", &devname) || !devname) {         /* ++ alloc device name */
+    devname = FLUID_STRDUP ("/dev/dsp");
+
+    if (devname == NULL) {
+      FLUID_LOG(FLUID_ERR, "Out of memory");
+      goto error_recovery;
+    }
   }
 
   if (stat(devname, &devstat) == -1) {
@@ -224,50 +221,18 @@ new_fluid_oss_audio_driver(fluid_settings_t* settings, fluid_synth_t* synth)
     goto error_recovery;
   }
 
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize audio thread attributes");
+  /* Create the audio thread */
+  dev->thread = new_fluid_thread (fluid_oss_audio_run, dev, realtime_prio, FALSE);
+
+  if (!dev->thread)
     goto error_recovery;
-  }
 
-  /* the pthread_create man page explains that
-     pthread_attr_setschedpolicy returns an error if the user is not
-     permitted the set SCHED_FIFO. it seems however that no error is
-     returned but pthread_create fails instead. that's why i try to
-     create the thread twice in a while loop. */
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy.");
-	goto error_recovery;
-      }
-    }
-
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? OSS_PCM_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam (&attr, &priority);
-
-    err = pthread_create(&dev->thread, &attr, fluid_oss_audio_run, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the audio thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
+  if (devname) FLUID_FREE (devname);    /* -- free device name */
 
   return (fluid_audio_driver_t*) dev;
 
 error_recovery:
+  if (devname) FLUID_FREE (devname);    /* -- free device name */
   delete_fluid_oss_audio_driver((fluid_audio_driver_t*) dev);
   return NULL;
 }
@@ -281,12 +246,9 @@ new_fluid_oss_audio_driver2(fluid_settings_t* settings, fluid_audio_func_t func,
   int queuesize;
   double sample_rate;
   int periods, period_size;
-  char* devname;
+  char* devname = NULL;
+  int realtime_prio = 0;
   int format;
-  pthread_attr_t attr;
-  int err;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
 
   dev = FLUID_NEW(fluid_oss_audio_driver_t);
   if (dev == NULL) {
@@ -298,6 +260,7 @@ new_fluid_oss_audio_driver2(fluid_settings_t* settings, fluid_audio_func_t func,
   fluid_settings_getint(settings, "audio.periods", &periods);
   fluid_settings_getint(settings, "audio.period-size", &period_size);
   fluid_settings_getnum(settings, "synth.sample-rate", &sample_rate);
+  fluid_settings_getint (settings, "audio.realtime-prio", &realtime_prio);
 
   dev->dspfd = -1;
   dev->synth = NULL;
@@ -310,9 +273,16 @@ new_fluid_oss_audio_driver2(fluid_settings_t* settings, fluid_audio_func_t func,
   dev->buffer_byte_size = dev->buffer_size * 2 * 2; /* 2 channels * 16 bits audio */
 
 
-  if (!fluid_settings_getstr(settings, "audio.oss.device", &devname)) {
-    devname = "/dev/dsp";
+  if (!fluid_settings_dupstr(settings, "audio.oss.device", &devname) || !devname) {
+    devname = FLUID_STRDUP ("/dev/dsp");
+
+    if (!devname)
+    {
+      FLUID_LOG(FLUID_ERR, "Out of memory");
+      goto error_recovery;
+    }
   }
+
   if (stat(devname, &devstat) == -1) {
     FLUID_LOG(FLUID_ERR, "Device <%s> does not exists", devname);
     goto error_recovery;
@@ -375,50 +345,18 @@ new_fluid_oss_audio_driver2(fluid_settings_t* settings, fluid_audio_func_t func,
     goto error_recovery;
   }
 
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize audio thread attributes");
+  /* Create the audio thread */
+  dev->thread = new_fluid_thread (fluid_oss_audio_run2, dev, realtime_prio, FALSE);
+
+  if (!dev->thread)
     goto error_recovery;
-  }
 
-  /* the pthread_create man page explains that
-     pthread_attr_setschedpolicy returns an error if the user is not
-     permitted the set SCHED_FIFO. it seems however that no error is
-     returned but pthread_create fails instead. that's why i try to
-     create the thread twice in a while loop. */
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy.");
-	goto error_recovery;
-      }
-    }
-
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? OSS_PCM_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam (&attr, &priority);
-
-    err = pthread_create(&dev->thread, &attr, fluid_oss_audio_run2, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the audio output");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the audio thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
+  if (devname) FLUID_FREE (devname);    /* -- free device name */
 
   return (fluid_audio_driver_t*) dev;
 
 error_recovery:
+  if (devname) FLUID_FREE (devname);    /* -- free device name */
   delete_fluid_oss_audio_driver((fluid_audio_driver_t*) dev);
   return NULL;
 }
@@ -434,13 +372,12 @@ delete_fluid_oss_audio_driver(fluid_audio_driver_t* p)
   if (dev == NULL) {
     return FLUID_OK;
   }
+
   dev->cont = 0;
-  if (dev->thread) {
-    if (pthread_join(dev->thread, NULL)) {
-      FLUID_LOG(FLUID_ERR, "Failed to join the audio thread");
-      return FLUID_FAILED;
-    }
-  }
+
+  if (dev->thread)
+    fluid_thread_join (dev->thread);
+
   if (dev->dspfd >= 0) {
     close(dev->dspfd);
   }
@@ -451,87 +388,6 @@ delete_fluid_oss_audio_driver(fluid_audio_driver_t* p)
   return FLUID_OK;
 }
 
-/*
- * fluid_oss_get_sample_formats
- */
-int
-fluid_oss_get_sample_formats(fluid_oss_audio_driver_t* dev)
-{
-  int mask;
-  unsigned short U16 = 1;
-  unsigned char* U8 = (unsigned char*) &U16;
-
-  dev->formats = 0;
-  dev->bigendian = 0;
-  if (ioctl(dev->dspfd, SNDCTL_DSP_GETFMTS, &mask) == -1) {
-    return -1;
-  }
-  dev->formats = mask;
-  if (U8[1] == 1) {
-    FLUID_LOG(FLUID_DBG, "Machine is big endian.");
-    dev->bigendian = 1;
-  }
-  if (U8[0] == 1) {
-    FLUID_LOG(FLUID_DBG, "Machine is little endian.");
-    dev->bigendian = 0;
-  }
-  FLUID_LOG(FLUID_DBG, "The sound device supports the following audio formats:");
-  if (mask & AFMT_U8)        { FLUID_LOG(FLUID_DBG, "  U8"); }
-  if (mask & AFMT_S8)        { FLUID_LOG(FLUID_DBG, "  S8"); }
-  if (mask & AFMT_U16_LE)    { FLUID_LOG(FLUID_DBG, "  U16LE"); }
-  if (mask & AFMT_U16_BE)    { FLUID_LOG(FLUID_DBG, "  U16BE"); }
-  if (mask & AFMT_S16_LE)    { FLUID_LOG(FLUID_DBG, "  S16LE"); }
-  if (mask & AFMT_S16_BE)    { FLUID_LOG(FLUID_DBG, "  S16BE"); }
-  if (mask & AFMT_MU_LAW)    { FLUID_LOG(FLUID_DBG, "  mu-law"); }
-  if (mask & AFMT_A_LAW)     { FLUID_LOG(FLUID_DBG, "  a-law"); }
-  if (mask & AFMT_IMA_ADPCM) { FLUID_LOG(FLUID_DBG, "  ima-adpcm"); }
-  if (mask & AFMT_MPEG)      { FLUID_LOG(FLUID_DBG, "  mpeg"); }
-  return 0;
-}
-
-/**
- *  fluid_oss_get_caps
- *
- *  Get the audio capacities of the sound card.
- */
-int
-fluid_oss_get_caps(fluid_oss_audio_driver_t* dev)
-{
-  int caps;
-  dev->caps = 0;
-  if (ioctl(dev->dspfd, SNDCTL_DSP_GETCAPS, &caps) < 0) {
-    return -1;
-  }
-  dev->caps = caps;
-  FLUID_LOG(FLUID_DBG, "The sound device has the following capabilities:");
-  if (caps & DSP_CAP_DUPLEX)   {
-    FLUID_LOG(FLUID_DBG, "  Duplex:    simultaneous playing and recording possible") ;
-  } else {
-    FLUID_LOG(FLUID_DBG, "  Duplex:    simultaneous playing and recording not possible");
-  }
-  if (caps & DSP_CAP_REALTIME) {
-    FLUID_LOG(FLUID_DBG, "  Real-time: precise reporting of output pointer possible");
-  } else {
-    FLUID_LOG(FLUID_DBG, "  Real-time: precise reporting of output pointer not possible");
-  }
-  if (caps & DSP_CAP_BATCH) {
-    FLUID_LOG(FLUID_DBG, "  Batch:     local storage for recording and/or playback");
-  } else {
-    FLUID_LOG(FLUID_DBG, "  Batch:     no local storage for recording and/or playback");
-  }
-  if (caps & DSP_CAP_TRIGGER) {
-    FLUID_LOG(FLUID_DBG, "  Trigger:   triggering of recording/playback possible");
-  } else {
-    FLUID_LOG(FLUID_DBG, "  Trigger:   triggering of recording/playback not possible");
-  }
-  if (caps & DSP_CAP_MMAP) {
-    FLUID_LOG(FLUID_DBG, "  Mmap:      direct access to the hardware level buffer possible");
-  } else {
-    FLUID_LOG(FLUID_DBG, "  Mmap:      direct access to the hardware level buffer not possible");
-  }
-  return 0;
-}
-
 /**
  *  fluid_oss_set_queue_size
  *
@@ -581,7 +437,7 @@ fluid_oss_set_queue_size(fluid_oss_audio_driver_t* dev, int ss, int ch, int qs,
 /*
  * fluid_oss_audio_run
  */
-void*
+void
 fluid_oss_audio_run(void* d)
 {
   fluid_oss_audio_driver_t* dev = (fluid_oss_audio_driver_t*) d;
@@ -593,21 +449,23 @@ fluid_oss_audio_run(void* d)
   while (dev->cont)
   {
     dev->read (synth, len, buffer, 0, 2, buffer, 1, 2);
-    write (dev->dspfd, buffer, dev->buffer_byte_size);
+
+    if (write (dev->dspfd, buffer, dev->buffer_byte_size) < 0)
+    {
+      FLUID_LOG(FLUID_ERR, "Error writing to OSS sound device: %s",
+                g_strerror (errno));
+      break;
+    }
   }
 
   FLUID_LOG(FLUID_DBG, "Audio thread finished");
-
-  pthread_exit(NULL);
-
-  return 0; /* not reached */
 }
 
 
 /*
  * fluid_oss_audio_run
  */
-void*
+void
 fluid_oss_audio_run2(void* d)
 {
   fluid_oss_audio_driver_t* dev = (fluid_oss_audio_driver_t*) d;
@@ -615,9 +473,7 @@ fluid_oss_audio_run2(void* d)
   float* left = dev->buffers[0];
   float* right = dev->buffers[1];
   int buffer_size = dev->buffer_size;
-  int len = dev->buffer_size;
   int dither_index = 0;
-  int i, k;
 
   FLUID_LOG(FLUID_DBG, "Audio thread running");
 
@@ -629,14 +485,15 @@ fluid_oss_audio_run2(void* d)
     fluid_synth_dither_s16 (&dither_index, buffer_size, left, right,
 			    buffer, 0, 2, buffer, 1, 2);
 
-    write (dev->dspfd, buffer, dev->buffer_byte_size);
+    if (write (dev->dspfd, buffer, dev->buffer_byte_size) < 0)
+    {
+      FLUID_LOG(FLUID_ERR, "Error writing to OSS sound device: %s",
+                g_strerror (errno));
+      break;
+    }
   }
 
   FLUID_LOG(FLUID_DBG, "Audio thread finished");
-
-  pthread_exit(NULL);
-
-  return 0; /* not reached */
 }
 
 
@@ -652,12 +509,9 @@ fluid_midi_driver_t*
 new_fluid_oss_midi_driver(fluid_settings_t* settings,
 			 handle_midi_event_func_t handler, void* data)
 {
-  int err;
   fluid_oss_midi_driver_t* dev;
-  pthread_attr_t attr;
-  int sched = SCHED_FIFO;
-  struct sched_param priority;
-  char* device;
+  int realtime_prio = 0;
+  char* device = NULL;
 
   /* not much use doing anything */
   if (handler == NULL) {
@@ -685,11 +539,20 @@ new_fluid_oss_midi_driver(fluid_settings_t* settings,
   }
 
   /* get the device name. if none is specified, use the default device. */
-  fluid_settings_getstr(settings, "midi.oss.device", &device);
+  fluid_settings_dupstr(settings, "midi.oss.device", &device);  /* ++ alloc device name */
+
   if (device == NULL) {
-    device = "/dev/midi";
+    device = FLUID_STRDUP ("/dev/midi");
+
+    if (!device)
+    {
+      FLUID_LOG(FLUID_ERR, "Out of memory");
+      goto error_recovery;
+    }
   }
 
+  fluid_settings_getint (settings, "midi.realtime-prio", &realtime_prio);
+
   /* open the default hardware device. only use midi in. */
   dev->fd = open(device, O_RDONLY, 0);
   if (dev->fd < 0) {
@@ -697,47 +560,27 @@ new_fluid_oss_midi_driver(fluid_settings_t* settings,
     goto error_recovery;
   }
 
-  dev->status = FLUID_MIDI_READY;
-
-  /* create the midi thread */
-  if (pthread_attr_init(&attr)) {
-    FLUID_LOG(FLUID_ERR, "Couldn't initialize midi thread attributes");
+  if (fcntl (dev->fd, F_SETFL, O_NONBLOCK) == -1)
+  {
+    FLUID_LOG(FLUID_ERR, "Failed to set OSS MIDI device to non-blocking: %s",
+              strerror (errno));
     goto error_recovery;
   }
-  /* use fifo scheduling. if it fails, use default scheduling. */
-  while (1) {
-    err = pthread_attr_setschedpolicy(&attr, sched);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_ERR, "Couldn't set scheduling policy");
-	goto error_recovery;
-      }
-    }
 
-    /* SCHED_FIFO will not be active without setting the priority */
-    priority.sched_priority = (sched == SCHED_FIFO) ? OSS_MIDI_SCHED_PRIORITY : 0;
-    pthread_attr_setschedparam (&attr, &priority);
+  dev->status = FLUID_MIDI_READY;
+
+  /* create MIDI thread */
+  dev->thread = new_fluid_thread (fluid_oss_midi_run, dev, realtime_prio, FALSE);
+
+  if (!dev->thread)
+    goto error_recovery;
+
+  if (device) FLUID_FREE (device);      /* ++ free device */
 
-    err = pthread_create(&dev->thread, &attr, fluid_oss_midi_run, (void*) dev);
-    if (err) {
-      FLUID_LOG(FLUID_WARN, "Couldn't set high priority scheduling for the MIDI input");
-      if (sched == SCHED_FIFO) {
-	sched = SCHED_OTHER;
-	continue;
-      } else {
-	FLUID_LOG(FLUID_PANIC, "Couldn't create the midi thread.");
-	goto error_recovery;
-      }
-    }
-    break;
-  }
   return (fluid_midi_driver_t*) dev;
 
  error_recovery:
+  if (device) FLUID_FREE (device);      /* ++ free device */
   delete_fluid_oss_midi_driver((fluid_midi_driver_t*) dev);
   return NULL;
 }
@@ -748,7 +591,6 @@ new_fluid_oss_midi_driver(fluid_settings_t* settings,
 int
 delete_fluid_oss_midi_driver(fluid_midi_driver_t* p)
 {
-  int err;
   fluid_oss_midi_driver_t* dev;
 
   dev = (fluid_oss_midi_driver_t*) p;
@@ -756,20 +598,12 @@ delete_fluid_oss_midi_driver(fluid_midi_driver_t* p)
     return FLUID_OK;
   }
 
+  /* cancel the thread and wait for it before cleaning up */
   dev->status = FLUID_MIDI_DONE;
 
-  /* cancel the thread and wait for it before cleaning up */
-  if (dev->thread) {
-    err = pthread_cancel(dev->thread);
-    if (err) {
-      FLUID_LOG(FLUID_ERR, "Failed to cancel the midi thread");
-      return FLUID_FAILED;
-    }
-    if (pthread_join(dev->thread, NULL)) {
-      FLUID_LOG(FLUID_ERR, "Failed to join the midi thread");
-      return FLUID_FAILED;
-    }
-  }
+  if (dev->thread)
+    fluid_thread_join (dev->thread);
+
   if (dev->fd >= 0) {
     close(dev->fd);
   }
@@ -783,34 +617,42 @@ delete_fluid_oss_midi_driver(fluid_midi_driver_t* p)
 /*
  * fluid_oss_midi_run
  */
-void*
+void
 fluid_oss_midi_run(void* d)
 {
-  int n, i;
-  fluid_midi_event_t* evt;
   fluid_oss_midi_driver_t* dev = (fluid_oss_midi_driver_t*) d;
-
-  /* make sure the other threads can cancel this thread any time */
-  if (pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
-  if (pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL)) {
-    FLUID_LOG(FLUID_ERR, "Failed to set the cancel state of the midi thread");
-    pthread_exit(NULL);
-  }
+  fluid_midi_event_t* evt;
+  struct pollfd fds;
+  int n, i;
 
   /* go into a loop until someone tells us to stop */
   dev->status = FLUID_MIDI_LISTENING;
 
+  fds.fd = dev->fd;
+  fds.events = POLLIN;
+  fds.revents = 0;
+
   while (dev->status == FLUID_MIDI_LISTENING) {
 
+    n = poll (&fds, 1, 100);
+
+    if (n == 0) continue;
+
+    if (n < 0)
+    {
+      FLUID_LOG(FLUID_ERR, "Error waiting for MIDI input: %s", strerror (errno));
+      break;
+    }
+
     /* read new data */
     n = read(dev->fd, dev->buffer, BUFFER_LENGTH);
+
+    if (n == -EAGAIN) continue;
+
     if (n < 0) {
       perror("read");
       FLUID_LOG(FLUID_ERR, "Failed to read the midi input");
-      dev->status = FLUID_MIDI_DONE;
+      break;
     }
 
     /* let the parser convert the data into events */
@@ -821,9 +663,7 @@ fluid_oss_midi_run(void* d)
 	(*dev->driver.handler)(dev->driver.data, evt);
       }
     }
-
   }
-  pthread_exit(NULL);
 }
 
 int
