@@ -1376,7 +1376,7 @@ fluid_settings_foreach_option (fluid_settings_t* settings, const char *name,
   newlist = fluid_list_sort (newlist, fluid_list_str_compare_func);
 
   for (p = newlist; p; p = p->next)
-    (*func)(data, name, (const char *)fluid_list_get (p));
+    (*func)(data, (char *)name, (char *)fluid_list_get (p));
 
   fluid_rec_mutex_unlock (settings->mutex);   /* -- unlock */
 
@@ -1564,7 +1564,7 @@ fluid_settings_foreach (fluid_settings_t* settings, void* data,
   for (p = bag.names; p; p = p->next)
   {
     r = fluid_settings_get (settings, (char *)(p->data), &node);
-    if (r && node) (*func) (data, (const char *)(p->data), node->type);
+    if (r && node) (*func) (data, (char *)(p->data), node->type);
     FLUID_FREE (p->data);       /* -- Free name */
   }
 
