@@ -48,6 +48,8 @@
 #define FLUID_NUM_PROGRAMS      128
 #define DRUM_INST_BANK		128
 
+#define FLUID_UNSET_PROGRAM     128     /* Program number used to unset a preset */
+
 #if defined(WITH_FLOAT)
 #define FLUID_SAMPLE_FORMAT     FLUID_SAMPLE_FLOAT
 #else
@@ -123,7 +125,6 @@ typedef struct _fluid_sfont_info_t {
  * sfont_info<>
  * tuning
  * sfont_id
- * gain
  * reverb_roomsize, reverb_damping, reverb_width, reverb_level
  * chorus_nr, chorus_level, chorus_speed, chorus_depth, chorus_type
  *
@@ -132,6 +133,7 @@ typedef struct _fluid_sfont_info_t {
  * with_reverb
  * with_chorus
  * state
+ * gain
  * cpu_load
  * noteid
  * storeid
@@ -146,7 +148,6 @@ typedef struct _fluid_sfont_info_t {
  * cur
  * dither_index
  * polyphony
- * st_gain
  * active_voice_count
  */
 
@@ -159,11 +160,14 @@ struct _fluid_synth_t
   fluid_rec_mutex_t mutex;           /**< Lock for multi-thread sensitive variables (not used by synthesis process) */
   fluid_list_t *queue_pool;          /**< List of event queues whose threads have been destroyed and which can be re-used */
   fluid_event_queue_t *return_queue; /**< Event queue for events from synthesis thread to non-synthesis threads (memory frees, etc) */
-  fluid_timer_t *return_queue_timer; /**< Timer thread to process return event queue */
+  fluid_thread_t *return_queue_thread;  /**< Return event queue processing thread */
+  fluid_cond_mutex_t *return_queue_mutex;       /**< Mutex for return queue condition */
+  fluid_cond_t *return_queue_cond;   /**< Return queue thread synchronization condition */
 
   fluid_settings_t* settings;        /**< the synthesizer settings */
   int device_id;                     /**< Device ID used for SYSEX messages */
-  int polyphony;                     /**< maximum polyphony */
+  int polyphony;                     /**< Maximum polyphony */
+  int shadow_polyphony;              /**< Maximum polyphony shadow value (for non-synth threads) */
   int with_reverb;                  /**< Should the synth use the built-in reverb unit? */
   int with_chorus;                  /**< Should the synth use the built-in chorus unit? */
   int verbose;                      /**< Turn verbose mode on? */
@@ -183,8 +187,7 @@ struct _fluid_synth_t
   fluid_hashtable_t *sfont_hash;     /**< Hash of fluid_sfont_t->fluid_sfont_info_t (remains until SoundFont is deleted) */
   unsigned int sfont_id;             /**< Incrementing ID assigned to each loaded SoundFont */
 
-  double gain;                       /**< master gain */
-  double st_gain;                    /**< Synth thread gain shadow value */
+  float gain;                        /**< master gain */
   fluid_channel_t** channel;         /**< the channels */
   int nvoice;                        /**< the length of the synthesis process array (max polyphony allowed) */
   fluid_voice_t** voice;             /**< the synthesis voices */
