@@ -131,7 +131,7 @@ fluid_set_log_function(int level, fluid_log_function_t fun, void* data)
  * @param data User supplied data (not used)
  */
 void
-fluid_default_log_function(int level, const char* message, void* data)
+fluid_default_log_function(int level, char* message, void* data)
 {
   FILE* out;
 
@@ -736,10 +736,14 @@ delete_fluid_timer (fluid_timer_t *timer)
 int
 fluid_timer_join (fluid_timer_t *timer)
 {
+  int auto_destroy;
+
   if (timer->thread)
   {
+    auto_destroy = timer->auto_destroy;
     fluid_thread_join (timer->thread);
-    timer->thread = NULL;
+
+    if (!auto_destroy) timer->thread = NULL;
   }
 
   return FLUID_OK;
