@@ -22,6 +22,8 @@
 #ifndef _FLUIDSYNTH_PRIV_H
 #define _FLUIDSYNTH_PRIV_H
 
+#include <glib.h>
+
 #if HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -111,6 +113,8 @@
 #endif
 
 #if HAVE_WINDOWS_H
+#include <winsock2.h>
+#include <ws2tcpip.h>
 #include <windows.h>
 #endif
 
@@ -126,7 +130,6 @@
 #define STDIN_FILENO 0
 #define STDOUT_FILENO 1
 #define STDERR_FILENO 2
-#define WITHOUT_SERVER 1
 
 #endif
 
@@ -153,12 +156,6 @@ typedef double fluid_real_t;
 #endif
 
 
-typedef enum {
-  FLUID_OK = 0,
-  FLUID_FAILED = -1
-} fluid_status;
-
-
 #if defined(WIN32)
 typedef SOCKET fluid_socket_t;
 #else
@@ -168,57 +165,14 @@ typedef int fluid_socket_t;
 
 
 /** Integer types  */
-
-#if defined(MINGW32)
-
-/* Windows using MinGW32 */
-typedef int8_t             sint8;
-typedef uint8_t            uint8;
-typedef int16_t            sint16;
-typedef uint16_t           uint16;
-typedef int32_t            sint32;
-typedef uint32_t           uint32;
-typedef int64_t            sint64;
-typedef uint64_t           uint64;
-
-#elif defined(_WIN32)
-
-/* Windows */
-typedef signed __int8      sint8;
-typedef unsigned __int8    uint8;
-typedef signed __int16     sint16;
-typedef unsigned __int16   uint16;
-typedef signed __int32     sint32;
-typedef unsigned __int32   uint32;
-typedef signed __int64     sint64;
-typedef unsigned __int64   uint64;
-
-#elif defined(MACOS9)
-
-/* Macintosh */
-typedef signed char        sint8;
-typedef unsigned char      uint8;
-typedef signed short       sint16;
-typedef unsigned short     uint16;
-typedef signed int         sint32;
-typedef unsigned int       uint32;
-/* FIXME: needs to be verified */
-typedef long long          sint64;
-typedef unsigned long long uint64;
-
-#else
-
-/* Linux & Darwin */
-typedef int8_t             sint8;
-typedef u_int8_t           uint8;
-typedef int16_t            sint16;
-typedef u_int16_t          uint16;
-typedef int32_t            sint32;
-typedef u_int32_t          uint32;
-typedef int64_t            sint64;
-typedef u_int64_t          uint64;
-
-#endif
+typedef gint8              sint8;
+typedef guint8             uint8;
+typedef gint16             sint16;
+typedef guint16            uint16;
+typedef gint32             sint32;
+typedef guint32            uint32;
+typedef gint64             sint64;
+typedef guint64            uint64;
 
 
 /***************************************************************
@@ -232,13 +186,19 @@ typedef struct _fluid_tuning_t fluid_tuning_t;
 typedef struct _fluid_hashtable_t  fluid_hashtable_t;
 typedef struct _fluid_client_t fluid_client_t;
 typedef struct _fluid_server_socket_t fluid_server_socket_t;
+typedef struct _fluid_sample_timer_t fluid_sample_timer_t;
 
 /***************************************************************
  *
  *                      CONSTANTS
  */
 
-#define FLUID_BUFSIZE                64
+#define FLUID_BUFSIZE                64         /**< FluidSynth internal buffer size (in samples) */
+#define FLUID_MAX_EVENTS_PER_BUFSIZE 1024       /**< Maximum queued MIDI events per #FLUID_BUFSIZE */
+#define FLUID_MAX_RETURN_EVENTS      1024       /**< Maximum queued synthesis thread return events */
+#define FLUID_MAX_EVENT_QUEUES       16         /**< Maximum number of unique threads queuing events */
+#define FLUID_DEFAULT_AUDIO_RT_PRIO  60         /**< Default setting for audio.realtime-prio */
+#define FLUID_DEFAULT_MIDI_RT_PRIO   50         /**< Default setting for midi.realtime-prio */
 
 #ifndef PI
 #define PI                          3.141592654
@@ -265,7 +225,9 @@ typedef FILE*  fluid_file;
 #define FLUID_STRCMP(_s,_t)          strcmp(_s,_t)
 #define FLUID_STRNCMP(_s,_t,_n)      strncmp(_s,_t,_n)
 #define FLUID_STRCPY(_dst,_src)      strcpy(_dst,_src)
+#define FLUID_STRNCPY(_dst,_src,_n)  strncpy(_dst,_src,_n)
 #define FLUID_STRCHR(_s,_c)          strchr(_s,_c)
+#define FLUID_STRRCHR(_s,_c)         strrchr(_s,_c)
 #ifdef strdup
 #define FLUID_STRDUP(s)              strdup(s)
 #else
